#include <ap_int.h>
#include <cstdlib>

void load_weight_buffer_wino_6x6(
	ap_uint<128>* DDR,
	ap_uint<16> weight_buffer[9][1024],
	ap_uint<32> burst_length
)
{

    #ifdef __CSIM__
        if(burst_length%3!=0)
        {
            std::cerr<<"["<<__FILE__<<":line "<<__LINE__<<"]:"<<"Burst Length for weight should be multiples of 3"<<std::endl;
            exit(0);
        }
    #endif

#pragma HLS RESOURCE variable=weight_buffer core=RAM_T2P_BRAM
#pragma HLS array_partition variable=weight_buffer dim=1 factor=9

	ap_uint<2> counter=0;
	ap_uint<10> weight_address=0;
	
	for(ap_uint<32> ddr_address=0;ddr_address<burst_length;ddr_address++)
	{
#pragma HLS pipeline
		ap_uint<128> temp= DDR[ddr_address];

		ap_int<16> temp_array[8];
#pragma HLS array_partition variable=temp_array complete

		for(ap_uint<4> idx=0;idx<8;idx++)
		{
	#pragma HLS unroll factor=8
			temp_array[idx]= temp.range(idx*16+15,idx*16);
		}

		if(counter==0)
		{
			weight_buffer[0][weight_address]=temp_array[0];
			weight_buffer[1][weight_address]=temp_array[1];
			weight_buffer[2][weight_address]=temp_array[2];
			weight_buffer[3][weight_address]=temp_array[3];
			weight_buffer[4][weight_address]=temp_array[4];
			weight_buffer[5][weight_address]=temp_array[5];
			weight_buffer[6][weight_address]=temp_array[6];
			weight_buffer[7][weight_address]=temp_array[7];

		}
		else if(counter==1)
		{
			weight_buffer[8][weight_address]=temp_array[0];
			weight_buffer[0][weight_address+1]=temp_array[1];
			weight_buffer[1][weight_address+1]=temp_array[2];
			weight_buffer[2][weight_address+1]=temp_array[3];
			weight_buffer[3][weight_address+1]=temp_array[4];
			weight_buffer[4][weight_address+1]=temp_array[5];
			weight_buffer[5][weight_address+1]=temp_array[6];
			weight_buffer[6][weight_address+1]=temp_array[7];
		}
		else
		{
			weight_buffer[7][weight_address+1]=temp_array[0];
			weight_buffer[8][weight_address+1]=temp_array[1];
		}

		if(counter==2)
		{
			weight_address+=2;
			counter=0;
		}
		else counter++;
	}
}


void load_weight_buffer_wino_4x4(
	ap_uint<128>* DDR,
	ap_uint<16> weight_buffer[4][1024],
	ap_uint<32> burst_length
)
{
    #ifdef __CSIM__
        if(burst_length%2!=0)
        {
            std::cerr<<"["<<__FILE__<<":line "<<__LINE__<<"]:"<<"Burst Length for weight should be multiples of 2"<<std::endl;
            exit(0);
        }
    #endif

    #pragma HLS RESOURCE variable=weight_buffer core=RAM_T2P_BRAM
    #pragma HLS array_partition variable=weight_buffer dim=1 complete
    
    ap_uint<10> weight_address=0;
	
	for(ap_uint<32> ddr_address=0;ddr_address<burst_length;ddr_address++)
	{
        #pragma HLS pipeline
        ap_uint<128> DDR_data = DDR[ddr_address];
        

        weight_buffer[0][weight_address].range(15,0)=  DDR_data.range(0*16+15, 0*16);
        weight_buffer[1][weight_address].range(15,0)=  DDR_data.range(1*16+15, 1*16);
        weight_buffer[2][weight_address].range(15,0)=  DDR_data.range(2*16+15, 2*16);
        weight_buffer[3][weight_address].range(15,0)=  DDR_data.range(3*16+15, 3*16);
        weight_buffer[0][weight_address+1].range(15,0)=  DDR_data.range(4*16+15, 4*16);
        weight_buffer[1][weight_address+1].range(15,0)=  DDR_data.range(5*16+15, 5*16);
        weight_buffer[2][weight_address+1].range(15,0)=  DDR_data.range(6*16+15, 6*16);
        weight_buffer[3][weight_address+1].range(15,0)=  DDR_data.range(7*16+15, 7*16);

        weight_address+=2;    
    }
    
}




void read_input_ddr_to_buffer(
	ap_uint<16> input_buffer[8][16][1024]
)